from flask import make_response

from ..helpers import patched_wraps


def enabled_cors(
    allowed_headers=[
        "Origin",
        "X-Requested-With",
        "Content-Type",
        "Acccept",
        "Authorization",
    ],
    allowed_origin="*",
    allowed_methods=["GET", "POST", "HEAD", "DELETE", "OPTIONS"],
):
    allowed_headers_string = ", ".join(allowed_headers)
    allowed_methods_string = ",".join(allowed_methods)

    def decorator(func):
        @patched_wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            response = make_response(result)
            response.headers[
                "Access-Control-Allow-Headers"
            ] = allowed_headers_string
            response.headers["Access-Control-Allow-Origin"] = allowed_origin
            response.headers[
                "Access-Control-Allow-Methods"
            ] = allowed_methods_string
            return response

        return wrapper

    return decorator



from functools import WRAPPER_ASSIGNMENTS, wraps

from .utils import trailing_slash


def patched_wraps(func):
    assigned = WRAPPER_ASSIGNMENTS + (
        "view_class",
        "methods",
        "provide_automatic_options",
    )
    return wraps(func, assigned=assigned)


def configure_routes(app, handlers):
    for handler in handlers:
        app.add_url_rule(
            trailing_slash(
                handler.view_class.base_url + handler.view_class.route
            ),
            view_func=handler,
        )



import logging
from urllib.parse import urlparse

from flask import request
from flask.views import MethodView
from werkzeug.exceptions import MethodNotAllowed, NotFound
from werkzeug.routing import Map, Rule

logger = logging.getLogger(__name__)


class EndPointBase(MethodView):
    protected = True
    protected_endpoints = []

    base_url = None
    route = ""

    decorators = []

    def options(self, request, *args, **kwargs):
        return {"status": "ok"}

    def dispatch_request(self, *args, **kwargs):
        meth = getattr(self, request.method.lower(), None)

        if meth is None and request.method == "HEAD":
            meth = getattr(self, "get", None)

        if meth is None:
            raise MethodNotAllowed(
                description={
                    "title": "Method is not allowed",
                    "detail": "Only following methods are allowed: {}.".format(
                        ", ".join(list(self.methods))
                    ),
                }
            )

        if not args and kwargs.get("request") is None:
            kwargs.update({"request": request})

        if self.route and self.base_url not in request.url:
            server_name = urlparse(request.url_root).netloc
            rule = Rule(self.route)
            url_map = Map([rule], strict_slashes=False)
            matcher = url_map.bind(server_name)
            try:
                matched_data = matcher.match(request.path)
            except NotFound:
                raise NotFound(
                    description={
                        "title": (
                            "The requested URL was not found on the server"
                        ),
                        "detail": (
                            "If you entered the URL manually please check your \
                            spelling and try again."
                        ),
                    }
                )
            endpoint_arguments = matched_data[1]
            kwargs.update(endpoint_arguments)

        return meth(*args, **kwargs)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['gfra', 'gfra.decorators']

package_data = \
{'': ['*']}

install_requires = \
['flask>=1.0,<2.0']

setup_kwargs = {
    'name': 'gfra',
    'version': '0.1.0',
    'description': 'REST API framework for Google Serverless Functions',
    'long_description': '# gfra [![image](https://img.shields.io/pypi/v/gfra.svg)](https://python.org/pypi/gfra) [![image](https://img.shields.io/pypi/l/gfra.svg)](https://python.org/pypi/gfra) [![image](https://img.shields.io/pypi/pyversions/gfra.svg)](https://python.org/pypi/gfra)\n\n> REST API framework for Google Serverless Functions \n\n## Description\n\n**gfra** stands for **G**oogle **F**unctions **R**EST **A**PI\n\n## Usage\n\nWe use [poetry](https://github.com/sdispater/poetry) for dependency management and publishing.\n\n\n### Installation\n```\n$ poetry add gcloud-functions-rest-api\n```\n\n### Development\n\n```\nWIP\n```\n\n### Production\n```\nWIP\n```\n\n### Testing\n```\nWIP\n```\n\n## Documentation\nWIP\n\n## Contributions\n\nFeel free to send some [pull request](https://github.com/Tasyp/gfra/pulls) or [issue](https://github.com/Tasyp/gfra/issues).\n\n## License\nMIT license\n\n© 2018 [German Ivanov](https://github.com/Tasyp)',
    'author': 'German Ivanov',
    'author_email': 'germivanov@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
